@@ -5,29 +5,22 @@
 # This software may be used and distributed according to the terms of the
 # GNU General Public License version 2, incorporated herein by reference.
 
-import os
-
 from PyQt4.QtCore import *
 from PyQt4.QtGui import *
 
-from mercurial import error
-
-from tortoisehg.util import hglib, i18n
-from tortoisehg.hgqt.i18n import _
-from tortoisehg.hgqt import qtlib, cmdui
-
-keep = i18n.keepgettext()
+from tortoisehg.util import hglib
+from tortoisehg.util.i18n import _
+from tortoisehg.hgqt import cmdcore, qtlib
 
 class BookmarkDialog(QDialog):
-    showMessage = pyqtSignal(QString)
-    output = pyqtSignal(QString, QString)
-    makeLogVisible = pyqtSignal(bool)
 
-    def __init__(self, repo, rev, parent):
+    def __init__(self, repoagent, rev, parent=None):
         super(BookmarkDialog, self).__init__(parent)
         self.setWindowFlags(self.windowFlags() & \
                             ~Qt.WindowContextHelpButtonHint)
-        self.repo = repo
+        self._repoagent = repoagent
+        repo = repoagent.rawRepo()
+        self._cmdsession = cmdcore.nullCmdSession()
         self.rev = rev
         self.node = repo[rev].node()
 
@@ -97,36 +90,46 @@ class BookmarkDialog(QDialog):
         self.status = qtlib.StatusLabel()
         self.status.setContentsMargins(4, 2, 4, 4)
         self.layout().addWidget(self.status)
+        self._finishmsg = None
 
         # dialog setting
-        self.setWindowTitle(_('Bookmark - %s') % self.repo.displayname)
+        self.setWindowTitle(_('Bookmark - %s') % repoagent.displayName())
         self.setWindowIcon(qtlib.geticon('hg-bookmarks'))
 
-        self.cmd = cmdui.Runner(False, self)
-        self.cmd.output.connect(self.output)
-        self.cmd.makeLogVisible.connect(self.makeLogVisible)
-        self.cmd.commandFinished.connect(self.commandFinished)
-
         # prepare to show
         self.clear_status()
         self.refresh()
-        self.repo.repositoryChanged.connect(self.refresh)
+        self._repoagent.repositoryChanged.connect(self.refresh)
         self.bookmarkCombo.setFocus()
         self.bookmarkTextChanged()
 
+    @property
+    def repo(self):
+        return self._repoagent.rawRepo()
+
+    def _allBookmarks(self):
+        return map(hglib.tounicode, self.repo._bookmarks)
+
+    @pyqtSlot()
     def refresh(self):
         """ update display on dialog with recent repo data """
         # add bookmarks to drop-down list
-        marks = self.repo._bookmarks.keys()[:]
-        marks.sort(reverse=True)
         cur = self.bookmarkCombo.currentText()
         self.bookmarkCombo.clear()
-        for bookmark in marks:
-            self.bookmarkCombo.addItem(hglib.tounicode(bookmark))
+        self.bookmarkCombo.addItems(sorted(self._allBookmarks()))
         if cur:
             self.bookmarkCombo.setEditText(cur)
         else:
-            self.bookmarkTextChanged()
+            ctx = self.repo[self.rev]
+            cs_bookmarks = ctx.bookmarks()
+            if self.repo._bookmarkcurrent in cs_bookmarks:
+                bm = hglib.tounicode(self.repo._bookmarkcurrent)
+                self.bookmarkCombo.setEditText(bm)
+            elif cs_bookmarks:
+                bm = hglib.tounicode(cs_bookmarks[0])
+                self.bookmarkCombo.setEditText(bm)
+            else:
+                self.bookmarkTextChanged()
 
     @pyqtSlot()
     def bookmarkTextChanged(self):
@@ -146,98 +149,84 @@ class BookmarkDialog(QDialog):
             self.renameBtn.setEnabled(False)
             self.newNameEdit.setEnabled(False)
 
+    def setBookmarkName(self, name):
+        self.bookmarkCombo.setEditText(name)
+
     def set_status(self, text, icon=None):
         self.status.setShown(True)
         self.sep.setShown(True)
         self.status.set_status(text, icon)
-        self.showMessage.emit(text)
 
     def clear_status(self):
         self.status.setHidden(True)
         self.sep.setHidden(True)
 
-    def commandFinished(self, ret):
-        if ret is 0:
+    def _runBookmark(self, *args, **opts):
+        self._finishmsg = opts.pop('finishmsg')
+        cmdline = hglib.buildcmdargs('bookmarks', *args, **opts)
+        self._cmdsession = sess = self._repoagent.runCommand(cmdline, self)
+        sess.commandFinished.connect(self._onBookmarkFinished)
+
+    @pyqtSlot(int)
+    def _onBookmarkFinished(self, ret):
+        if ret == 0:
             self.bookmarkCombo.clearEditText()
             self.newNameEdit.setText('')
-            self.finishfunc()
+            self.set_status(self._finishmsg, True)
+        else:
+            self.set_status(self._cmdsession.errorString(), False)
 
+    @pyqtSlot()
     def add_bookmark(self):
-        bookmark = self.bookmarkCombo.currentText()
-        bookmarklocal = hglib.fromunicode(bookmark)
-        if bookmarklocal in self.repo._bookmarks:
+        bookmark = unicode(self.bookmarkCombo.currentText())
+        if bookmark in self._allBookmarks():
             self.set_status(_('A bookmark named "%s" already exists') %
                             bookmark, False)
             return
 
-        def finished():
-            self.set_status(_("Bookmark '%s' has been added") % bookmark, True)
-
-        cmdline = ['bookmarks', '--repository', self.repo.root]
+        finishmsg = _("Bookmark '%s' has been added") % bookmark
+        rev = None
         if not self.activateCheckBox.isChecked():
-            cmdline.extend(['--rev', str(self.rev)])
-        cmdline.append(bookmarklocal)
-        self.cmd.run(cmdline)
-        self.finishfunc = finished
+            rev = self.rev
+        self._runBookmark(bookmark, rev=rev, finishmsg=finishmsg)
 
+    @pyqtSlot()
     def move_bookmark(self):
-        bookmark = self.bookmarkCombo.currentText()
-        bookmarklocal = hglib.fromunicode(bookmark)
-        if bookmarklocal not in self.repo._bookmarks:
+        bookmark = unicode(self.bookmarkCombo.currentText())
+        if bookmark not in self._allBookmarks():
             self.set_status(_('Bookmark named "%s" does not exist') %
                             bookmark, False)
             return
 
-        def finished():
-            self.set_status(_("Bookmark '%s' has been moved") % bookmark, True)
-
-        cmdline = ['bookmarks', '--repository', self.repo.root]
+        finishmsg = _("Bookmark '%s' has been moved") % bookmark
+        rev = None
         if not self.activateCheckBox.isChecked():
-            cmdline.extend(['--rev', str(self.rev)])
-        cmdline.extend(['--force', bookmarklocal])
-        self.cmd.run(cmdline)
-        self.finishfunc = finished
+            rev = self.rev
+        self._runBookmark(bookmark, rev=rev, force=True, finishmsg=finishmsg)
 
+    @pyqtSlot()
     def remove_bookmark(self):
-        bookmark = self.bookmarkCombo.currentText()
-        bookmarklocal = hglib.fromunicode(bookmark)
-        if bookmarklocal not in self.repo._bookmarks:
+        bookmark = unicode(self.bookmarkCombo.currentText())
+        if bookmark not in self._allBookmarks():
             self.set_status(_("Bookmark '%s' does not exist") % bookmark, False)
             return
 
-        def finished():
-            self.set_status(_("Bookmark '%s' has been removed") % bookmark, True)
-
-        cmdline = ['bookmarks', '--repository', self.repo.root,
-                   '--delete', bookmarklocal]
-        self.cmd.run(cmdline)
-        self.finishfunc = finished
+        finishmsg = _("Bookmark '%s' has been removed") % bookmark
+        self._runBookmark(bookmark, delete=True, finishmsg=finishmsg)
 
+    @pyqtSlot()
     def rename_bookmark(self):
-        name = self.bookmarkCombo.currentText()
-        namelocal = hglib.fromunicode(name)
-
-        newname = self.newNameEdit.text()
-        newnamelocal = hglib.fromunicode(newname)
-        if namelocal not in self.repo._bookmarks:
+        name = unicode(self.bookmarkCombo.currentText())
+        if name not in self._allBookmarks():
             self.set_status(_("Bookmark '%s' does not exist") % name, False)
             return
 
-        if newnamelocal in self.repo._bookmarks:
+        newname = unicode(self.newNameEdit.text())
+        if newname in self._allBookmarks():
             self.set_status(_('A bookmark named "%s" already exists') %
                             newname, False)
             return
 
-        def finished():
-            self.set_status(_("Bookmark '%s' has been renamed to '%s'") %
-                            (name, newname), True)
-
-        cmdline = ['bookmarks', '--repository', self.repo.root,
-                   '--rename', namelocal, newnamelocal]
-        self.cmd.run(cmdline)
-        self.finishfunc = finished
-
-    def reject(self):
-        # prevent signals from reaching deleted objects
-        self.repo.repositoryChanged.disconnect(self.refresh)
-        super(BookmarkDialog, self).reject()
+        finishmsg = (_("Bookmark '%s' has been renamed to '%s'")
+                     % (name, newname))
+        self._runBookmark(name, newname, rename=True, finishmsg=finishmsg)
