@@ -12,14 +12,14 @@ from PyQt4.QtCore import *
 from PyQt4.QtGui import *
 
 from mercurial import error
-from mercurial.node import hex
 
-from tortoisehg.util import hglib, paths
-from tortoisehg.hgqt.i18n import _
-from tortoisehg.hgqt import qtlib, thgrepo
+from tortoisehg.util import hglib
+from tortoisehg.util.i18n import _
+from tortoisehg.hgqt import qtlib
 
 PANEL_DEFAULT = ('rev', 'summary', 'user', 'dateage', 'branch', 'close',
-                 'tags', 'graft', 'transplant', 'p4', 'svn', 'converted')
+                 'tags', 'graft', 'transplant', 'obsolete',
+                 'p4', 'svn', 'converted',)
 
 def create(repo, target=None, style=None, custom=None, **kargs):
     return Factory(repo, custom, style, target, **kargs)()
@@ -108,10 +108,13 @@ class SummaryInfo(object):
               'dateage': _('Date:'), 'branch': _('Branch:'),
               'close': _('Close:'),
               'tags': _('Tags:'), 'rawbranch': _('Branch:'),
-              'rawtags': _('Tags:'), 'graft': _('Graft:'),
+              'graft': _('Graft:'),
               'transplant': _('Transplant:'),
+              'obsolete': _('Obsolete state:'),
               'p4': _('Perforce:'), 'svn': _('Subversion:'),
-              'converted': _('Converted From:'), 'shortuser': _('User:')}
+              'converted': _('Converted From:'), 'shortuser': _('User:'),
+              'mqoriginalparent': _('Original Parent:')
+    }
 
     def __init__(self):
         pass
@@ -134,10 +137,11 @@ class SummaryInfo(object):
             elif item == 'desc':
                 return hglib.tounicode(ctx.description().replace('\0', ''))
             elif item == 'summary':
-                value = ctx.description().replace('\0', '').split('\n')[0]
-                if len(value) == 0:
+                summary = hglib.longsummary(
+                    ctx.description().replace('\0', ''))
+                if len(summary) == 0:
                     return None
-                return hglib.tounicode(value)[:80]
+                return summary
             elif item == 'user':
                 user = hglib.user(ctx)
                 if user:
@@ -167,7 +171,11 @@ class SummaryInfo(object):
                 value = self.get_data('rawbranch', *args)
                 if value:
                     repo = ctx._repo
-                    if ctx.node() not in repo.branchtags().values():
+                    try:
+                        if ctx.node() != repo.branchtip(ctx.branch()):
+                            return None
+                    except error.RepoLookupError:
+                        # ctx.branch() can be invalid for null or workingctx
                         return None
                     if value in repo.deadbranches:
                         return None
@@ -175,10 +183,8 @@ class SummaryInfo(object):
                 return None
             elif item == 'close':
                 return ctx.extra().get('close')
-            elif item == 'rawtags':
-                return hglib.getrawctxtags(ctx)
             elif item == 'tags':
-                return hglib.getctxtags(ctx)
+                return ctx.thgtags() or None
             elif item == 'graft':
                 extra = ctx.extra()
                 try:
@@ -195,6 +201,16 @@ class SummaryInfo(object):
                 except KeyError:
                     pass
                 return None
+            elif item == 'obsolete':
+                obsoletestate = []
+                if ctx.obsolete():
+                    obsoletestate.append('obsolete')
+                if ctx.extinct():
+                    obsoletestate.append('extinct')
+                obsoletestate += ctx.troubles()
+                if obsoletestate:
+                    return obsoletestate
+                return None
             elif item == 'p4':
                 extra = ctx.extra()
                 p4cl = extra.get('p4', None)
@@ -219,6 +235,16 @@ class SummaryInfo(object):
             elif item == 'ishead':
                 childbranches = [cctx.branch() for cctx in ctx.children()]
                 return ctx.branch() not in childbranches
+            elif item == 'mqoriginalparent':
+                target = ctx.thgmqoriginalparent()
+                if not target:
+                    return None
+                p1 = ctx.p1()
+                if p1 is not None and p1.hex() == target:
+                    return None
+                if target not in ctx._repo:
+                    return None
+                return target
             raise UnknownItem(item)
         if 'data' in custom and not kargs.get('usepreset', False):
             try:
@@ -262,7 +288,7 @@ class SummaryInfo(object):
                 if revnum is not None and revid is not None:
                     return '%s (%s)' % (revnum, revid)
                 return '%s' % revid
-            elif item in ('revid', 'graft', 'transplant'):
+            elif item in ('revid', 'graft', 'transplant', 'mqoriginalparent'):
                 return qtlib.markup(value, **mono)
             elif item in ('revnum', 'p4', 'close', 'converted'):
                 return str(value)
@@ -272,7 +298,7 @@ class SummaryInfo(object):
             elif item in ('rawbranch', 'branch'):
                 opts = dict(fg='black', bg='#aaffaa')
                 return qtlib.markup(' %s ' % value, **opts)
-            elif item in ('rawtags', 'tags'):
+            elif item == 'tags':
                 opts = dict(fg='black', bg='#ffffaa')
                 tags = [qtlib.markup(' %s ' % tag, **opts) for tag in value]
                 return ' '.join(tags)
@@ -281,6 +307,11 @@ class SummaryInfo(object):
                 return qtlib.markup(value)
             elif item == 'dateage':
                 return qtlib.markup('%s (%s)' % value)
+            elif item == 'obsolete':
+                opts = dict(fg='black', bg='#ff8566')
+                obsoletestates = [qtlib.markup(' %s ' % state, **opts)
+                                  for state in value]
+                return ' '.join(obsoletestates)
             raise UnknownItem(item)
         value = self.get_data(item, *args)
         if value is None:
@@ -387,16 +418,15 @@ class SummaryPanel(SummaryBase, QWidget):
 
         if self.revlabel is None:
             self.revlabel = QLabel()
-            self.revlabel.linkActivated.connect(
-                 lambda s: self.linkActivated.emit(s))
-            self.layout().addWidget(self.revlabel, alignment=Qt.AlignTop)
+            self.revlabel.linkActivated.connect(self.linkActivated)
+            self.layout().addWidget(self.revlabel, 0, Qt.AlignTop)
 
         if 'expandable' in self.csstyle and self.csstyle['expandable']:
             if self.expand_btn.parentWidget() is None:
                 self.expand_btn.clicked.connect(lambda: self.update())
                 margin = QHBoxLayout()
                 margin.setMargin(3)
-                margin.addWidget(self.expand_btn, alignment=Qt.AlignTop)
+                margin.addWidget(self.expand_btn, 0, Qt.AlignTop)
                 self.layout().insertLayout(0, margin)
             self.expand_btn.setShown(True)
         elif self.expand_btn.parentWidget() is not None:
