@@ -8,19 +8,18 @@
 # GNU General Public License version 2, incorporated herein by reference.
 
 import os, tempfile, re
-from StringIO import StringIO
 from PyQt4.QtCore import *
 from PyQt4.QtGui import *
-from mercurial import error, extensions, util, scmutil
-from tortoisehg.util import hglib, paths
-from tortoisehg.hgqt.i18n import _
-from tortoisehg.hgqt import cmdui, lexers, qtlib, thgrepo
+from mercurial import error, util
+from tortoisehg.util import hglib
+from tortoisehg.util.i18n import _
+from tortoisehg.hgqt import cmdcore, cmdui, lexers, qtlib
 from tortoisehg.hgqt.hgemail_ui import Ui_EmailDialog
 
 class EmailDialog(QDialog):
     """Dialog for sending patches via email"""
 
-    def __init__(self, repo, revs, parent=None, outgoing=False,
+    def __init__(self, repoagent, revs, parent=None, outgoing=False,
                  outgoingrevs=None):
         """Create EmailDialog for the given repo and revs
 
@@ -32,7 +31,8 @@ class EmailDialog(QDialog):
         """
         super(EmailDialog, self).__init__(parent)
         self.setWindowFlags(Qt.Window)
-        self._repo = repo
+        self._repoagent = repoagent
+        self._cmdsession = cmdcore.nullCmdSession()
         self._outgoing = outgoing
         self._outgoingrevs = outgoingrevs or []
 
@@ -97,17 +97,17 @@ class EmailDialog(QDialog):
             s.setValue('email/%s' % k, w.isChecked())
 
     def _initchangesets(self, revs):
-        def purerevs(revs):
-            return scmutil.revrange(self._repo, iter(str(e) for e in revs))
-
         self._changesets = _ChangesetsModel(self._repo,
-                                            # TODO: [':'] is inefficient
-                                            revs=purerevs(revs or [':']),
-                                            selectedrevs=purerevs(revs),
+                                            revs=revs or list(self._repo),
+                                            selectedrevs=revs,
                                             parent=self)
         self._changesets.dataChanged.connect(self._updateforms)
         self._qui.changesets_view.setModel(self._changesets)
 
+    @property
+    def _repo(self):
+        return self._repoagent.rawRepo()
+
     @property
     def _ui(self):
         return self._repo.ui
@@ -169,7 +169,7 @@ class EmailDialog(QDialog):
         """Generate opts for patchbomb by form values"""
         def headertext(s):
             # QLineEdit may contain newline character
-            return re.sub(r'\s', ' ', hglib.fromunicode(s))
+            return re.sub(r'\s', ' ', unicode(s))
 
         opts['to'] = [headertext(self._qui.to_edit.currentText())]
         opts['cc'] = [headertext(self._qui.cc_edit.currentText())]
@@ -179,10 +179,10 @@ class EmailDialog(QDialog):
 
         if self._qui.bundle_radio.isChecked():
             assert self._outgoing  # only outgoing bundle is supported
-            opts['rev'] = map(str, self._outgoingrevs)
+            opts['rev'] = hglib.compactrevs(self._outgoingrevs)
             opts['bundle'] = True
         else:
-            opts['rev'] = map(str, self._revs)
+            opts['rev'] = hglib.compactrevs(self._revs)
 
         def diffformat():
             n = self.getdiffformat()
@@ -195,22 +195,26 @@ class EmailDialog(QDialog):
         opts.update(self.getextraopts())
 
         def writetempfile(s):
-            fd, fname = tempfile.mkstemp(prefix='thg_emaildesc_')
+            fd, fname = tempfile.mkstemp(prefix='thg_emaildesc_',
+                                         dir=qtlib.gettempdir())
             try:
                 os.write(fd, s)
-                return fname
+                return hglib.tounicode(fname)
             finally:
                 os.close(fd)
 
         opts['intro'] = self._qui.writeintro_check.isChecked()
         if opts['intro']:
             opts['subject'] = headertext(self._qui.subject_edit.currentText())
-            opts['desc'] = writetempfile(hglib.fromunicode(self._qui.body_edit.toPlainText()))
-            # TODO: change patchbomb not to use temporary file
+            opts['desc'] = writetempfile(
+                hglib.fromunicode(self._qui.body_edit.toPlainText()))
 
-        # Include the repo in the command so it can be found when thg is not
-        # run from within a hg path
-        opts['repository'] = self._repo.root
+        # The email dialog is available no matter if patchbomb extension isn't
+        # enabled.  The extension name makes it unlikely first-time users
+        # would discover that Mercurial ships with a functioning patch MTA.
+        # Since patchbomb doesn't monkey patch any Mercurial code, it's safe
+        # to enable it on demand.
+        opts['config'] = 'extensions.patchbomb='
 
         return opts
 
@@ -220,7 +224,8 @@ class EmailDialog(QDialog):
             if not getattr(self._qui, e).currentText():
                 return False
 
-        if self._qui.writeintro_check.isChecked() and not self._qui.subject_edit.currentText():
+        if (self._qui.writeintro_check.isChecked()
+            and not self._qui.subject_edit.currentText()):
             return False
 
         if not self._revs:
@@ -247,7 +252,7 @@ class EmailDialog(QDialog):
         if self._introrequired():
             self._qui.writeintro_check.setChecked(True)
 
-    #@pyqtSlot()
+    @qtlib.senderSafeSlot()
     def _updateattachmodes(self):
         """Update checkboxes to select the embedding style of the patch"""
         attachmodes = [self._qui.attach_check, self._qui.inline_check]
@@ -269,32 +274,15 @@ class EmailDialog(QDialog):
             getattr(self._qui, e).editTextChanged.connect(self._updateforms)
 
     def accept(self):
-        # TODO: want to pass patchbombopts directly
-        def cmdargs(opts):
-            args = []
-            for k, v in opts.iteritems():
-                if isinstance(v, bool):
-                    if v:
-                        args.append('--%s' % k.replace('_', '-'))
-                else:
-                    for e in isinstance(v, basestring) and [v] or v:
-                        args += ['--%s' % k.replace('_', '-'), e]
-
-            return args
-
-        hglib.loadextension(self._ui, 'patchbomb')
-
         opts = self._patchbombopts()
-        try:
-            cmd = cmdui.Dialog(['email'] + cmdargs(opts), parent=self)
-            cmd.setWindowTitle(_('Sending Email'))
-            cmd.setShowOutput(False)
-            cmd.finished.connect(cmd.deleteLater)
-            if cmd.exec_():
-                self._writehistory()
-        finally:
-            if 'desc' in opts:
-                os.unlink(opts['desc'])  # TODO: don't use tempfile
+        cmdline = hglib.buildcmdargs('email', **opts)
+        cmd = cmdui.CmdSessionDialog(self)
+        cmd.setWindowTitle(_('Sending Email'))
+        cmd.setLogVisible(False)
+        uih = cmdui.PasswordUiHandler(cmd)  # skip "intro" prompt
+        cmd.setSession(self._repoagent.runCommand(cmdline, uih))
+        if cmd.exec_() == 0:
+            self._writehistory()
 
     def _initintrobox(self):
         self._qui.intro_box.hide()  # hidden by default
@@ -303,7 +291,7 @@ class EmailDialog(QDialog):
 
     def _introrequired(self):
         """Is intro message required?"""
-        return len(self._revs) > 1 or self._qui.bundle_radio.isChecked()
+        return self._qui.bundle_radio.isChecked()
 
     def _initpreviewtab(self):
         def initqsci(w):
@@ -316,7 +304,7 @@ class EmailDialog(QDialog):
             lex.setFont(fh.font())
             w.setLexer(lex)
             # TODO: better way to setup diff lexer
-          
+
         initqsci(self._qui.preview_edit)
 
         self._qui.main_tabs.currentChanged.connect(self._refreshpreviewtab)
@@ -332,40 +320,17 @@ class EmailDialog(QDialog):
         if self._previewtabindex() != index:
             return
 
-        self._qui.preview_edit.setText(self._preview())
-
-    def _preview(self):
-        """Generate preview text by running patchbomb"""
-        def loadpatchbomb():
-            hglib.loadextension(self._ui, 'patchbomb')
-            return extensions.find('patchbomb')
-
-        def wrapui(ui):
-            buf = StringIO()
-            # TODO: common way to prepare pure ui
-            newui = ui.copy()
-            newui.setconfig('ui', 'interactive', False)
-            newui.setconfig('diff', 'git', False)
-            newui.write = lambda *args, **opts: buf.write(''.join(args))
-            newui.status = lambda *args, **opts: None
-            return newui, buf
-
-        def stripheadmsg(s):
-            # TODO: skip until first Content-type: line ??
-            return '\n'.join(s.splitlines()[3:])
-
-        ui, buf = wrapui(self._ui)
+        self._qui.preview_edit.clear()
         opts = self._patchbombopts(test=True)
-        try:
-            # TODO: fix hgext.patchbomb's implementation instead
-            if 'PAGER' in os.environ:
-                del os.environ['PAGER']
+        cmdline = hglib.buildcmdargs('email', **opts)
+        self._cmdsession = sess = self._repoagent.runCommand(cmdline)
+        sess.setCaptureOutput(True)
+        sess.commandFinished.connect(self._updatepreview)
 
-            loadpatchbomb().patchbomb(ui, self._repo, **opts)
-            return stripheadmsg(hglib.tounicode(buf.getvalue()))
-        finally:
-            if 'desc' in opts:
-                os.unlink(opts['desc'])  # TODO: don't use tempfile
+    @pyqtSlot()
+    def _updatepreview(self):
+        msg = hglib.tounicode(str(self._cmdsession.readAll()))
+        self._qui.preview_edit.append(msg)
 
     def _previewtabindex(self):
         """Index of preview tab"""
@@ -388,7 +353,9 @@ class EmailDialog(QDialog):
     def on_selectnone_button_clicked(self):
         self._changesets.selectNone()
 
-class _ChangesetsModel(QAbstractTableModel):  # TODO: use component of log viewer?
+
+# TODO: use component of log viewer?
+class _ChangesetsModel(QAbstractTableModel):
     _COLUMNS = [('rev', lambda ctx: '%d:%s' % (ctx.rev(), ctx)),
                 ('author', lambda ctx: hglib.username(ctx.user())),
                 ('date', lambda ctx: util.shortdate(ctx.date())),
@@ -483,20 +450,3 @@ class _ChangesetsModel(QAbstractTableModel):  # TODO: use component of log viewe
         first = self.createIndex(0, 0)
         last = self.createIndex(len(self._revs) - 1, 0)
         self.dataChanged.emit(first, last)
-
-def run(ui, *revs, **opts):
-    # TODO: same options as patchbomb
-    if opts.get('rev'):
-        if revs:
-            raise util.Abort(_('use only one form to specify the revision'))
-        revs = opts.get('rev')
-
-    # TODO: repo should be a required argument?
-    repo = opts.get('repo') or thgrepo.repository(ui, paths.find_root())
-
-    try:
-        return EmailDialog(repo, revs, outgoing=opts.get('outgoing', False),
-                           outgoingrevs=opts.get('outgoingrevs', None))
-    except error.RepoLookupError, e:
-        qtlib.ErrorMsgBox(_('Failed to open Email dialog'),
-                          hglib.tounicode(e.message))
