@@ -11,7 +11,7 @@ from PyQt4.QtGui import *
 import os
 
 from tortoisehg.util import hglib
-from tortoisehg.hgqt.i18n import _
+from tortoisehg.util.i18n import _
 from tortoisehg.hgqt import qtlib, cmdui, csinfo, visdiff, thgrepo
 
 MARGINS = (8, 0, 0, 0)
